MODULE datasetmod

   implicit none

   type dataset_type
     real, dimension(:), allocatable :: x
     real, dimension(:), allocatable :: x0
     real, dimension(:), allocatable :: y
     real                            :: wave
     integer                         :: radtype
     character(len=256)              :: fname
     real                            :: tstep,tmin,tmax

   contains

    procedure :: open_file
    procedure :: set_data
    procedure :: save_bin
    procedure :: resize

   end type dataset_type
   

CONTAINS

   subroutine open_file(this,filename,ier,wavel,filetyp)
!
!  Load dataset from external file
!
   USE prog_constants
   USE filereading
   USE fileutil
   USE counts
   class(dataset_type), intent(inout)          :: this
   character(len=*), intent(in)                :: filename
   integer, intent(out)                        :: ier
   real, intent(in), optional                  :: wavel   ! wavel <= 0 will be ignored
   integer, intent(in), optional               :: filetyp
   integer                                     :: count_type
   integer                                     :: ftype
   real                                        :: thmino, thstepo, thmaxo, wavelen
   integer                                     :: ncount
   type(point_type), allocatable, dimension(:) :: countp
   type(error_type)                            :: error
!
   if (present(filetyp)) then
       ftype = filetyp
   else
       ftype = filetype_from_filename(filename)
   endif
   call read_datafile(filename,ftype,thmino,    &
        thstepo,thmaxo,countp,ncount,.false.,wavelen,count_type,error)
   if (error%signal) then
       call error%print()
       ier = 1
   else
       ier = 0
       this%radtype = 1
       this%tstep = thstepo
       this%tmin = thmino
       this%tmax = thmaxo
       if (present(wavel)) then
           if (wavel > 0) wavelen = wavel
       endif
       if (wavelen <= 0) then
           this%wave = DEF_WAVE
           call openwave(this%wave, this%radtype)
       else
           this%wave = wavelen
       endif
       if (count_type == COUNT_D_TYPE) countp = thvalue(countp,this%wave)
       call this%set_data(countp(:)%x,countp(:)%y,size(countp),filename)
   endif
   end subroutine open_file

!-------------------------------------------------------------------------

   subroutine set_data(this,x,y,np,fname,x0)
   USE arrayutil
   class(dataset_type), intent(inout) :: this
   real, dimension(:), intent(in)     :: x,y
   integer, intent(in)                :: np
   character(len=*), intent(in)       :: fname
   real, dimension(:), optional, intent(in) :: x0
   if (np > 0) then
!old       call reallocate(this%x,np,savevet=.false.)
!old       call reallocate(this%x0,np,savevet=.false.)
!old       call reallocate(this%y,np,savevet=.false.)
       call new_array(this%x,np)
       call new_array(this%x0,np)
       call new_array(this%y,np)
       this%x(:) = x(:np)
       this%y(:) = y(:np)
       this%tmin = this%x(1)
       this%tmax = this%x(np)
       if (present(x0)) then
           this%x0 = x0
       else
           this%x = x
       endif
   endif
   this%fname = fname
   end subroutine set_data

!-------------------------------------------------------------------------

   subroutine save_bin(this,unitbin)
   USE arrayutil
   class(dataset_type), intent(inout) :: this
   integer, intent(in)                :: unitbin
   integer :: np
!
   np = size_array(this%x) 
   write(unitbin) np
   if (np > 0) write(unitbin) this%x,this%x0,this%y,this%wave,this%tstep,this%tmin,this%tmax,this%radtype,this%fname
   !if (np > 0) write(unitbin) this   ! invalid I/O
!
   end subroutine save_bin

!-------------------------------------------------------------------------

   subroutine read_data_bin(this, unitbin, err)
   USE arrayutil
   USE errormod
   class(dataset_type), intent(inout) :: this
   integer, intent(in)                :: unitbin
   type(error_type), intent(out)      :: err
   integer                            :: ier
   integer                            :: np
   read(unitbin,iostat=ier,err=10) np
   if (np > 0) then
       call new_array(this%x,np)
       call new_array(this%x0,np)
       call new_array(this%y,np)
       read(unitbin,iostat=ier,err=10)this%x,this%x0,this%y,this%wave,this%tstep,this%tmin,this%tmax,this%radtype,this%fname
   endif
10 continue
   if (ier /= 0) then
       call err%set('Error on reading diffraction data')
   endif
   end subroutine read_data_bin

!-------------------------------------------------------------------------

   subroutine resize(this,xmin,xmax)
   USE arrayutil
   class(dataset_type), intent(inout) :: this
   real, intent(in), optional         :: xmin,xmax
   integer                            :: ipmin,ipmax,olddim,newdim
!
   olddim = size_array(this%x)
   if (olddim == 0) return

   if (present(xmin)) then
       ipmin = clocate(this%x,xmin)                  ! locazione del punto + vicino
       if (this%x(ipmin) < xmin) ipmin = ipmin + 1   ! ma non deve essere < di xmin
   else
       ipmin = 1
   endif
   if (present(xmax)) then
       ipmax = clocate(this%x,xmax)                ! locazione del punto + vicino
       if (this%x(ipmax) > xmax) ipmax = ipmax - 1 ! ma non deve essere > di xmax
   else
       ipmax = olddim
   endif
   newdim = ipmax - ipmin + 1
   if (newdim > 0 .and. newdim /= olddim) then
       this%x(1:newdim) = this%x(ipmin:ipmax)
       this%x0(1:newdim) = this%x0(ipmin:ipmax)
       this%y(1:newdim) = this%y(ipmin:ipmax)
       call resize_array(this%x,newdim)
       call resize_array(this%x0,newdim)
       call resize_array(this%y,newdim)
       this%tmin = this%x(1)
       this%tmax = this%x(newdim)
   endif
!
   end subroutine resize

END MODULE datasetmod
