MODULE crystal_phase
   USE reflection_type_util, only: reflection_type
   USE atom_basic, only: atom_type
   USE connect_mod, only: bond_type
   USE spginfom
   USE unit_cell
   USE elements

   implicit none

   type crystal_phase_type 
     type(atom_type), dimension(:), allocatable       :: at
     type(bond_type), dimension(:), allocatable       :: bonds
     type(spaceg_type)                                :: spg
     type(cell_type)                                  :: cell
     type(element_type), dimension(:), allocatable    :: elem
     type(reflection_type), dimension(:), allocatable :: ref
     character(len=100)                               :: phase_name = ' '
     real                                             :: scal = 1.0
     real, dimension(:), allocatable                  :: yc

     logical, private                                 :: has_cell = .false.
     logical, private                                 :: has_spg = .false.
   contains

     ! Data modification methods
     procedure :: set => set_crystal
     procedure :: set_atoms
     procedure :: set_bonds
     procedure :: set_symmetry
     procedure :: set_reflections
     procedure :: set_scattering

     procedure, private :: set_spg
     procedure, private :: set_unit_cell

     ! Data retrieval methods
     procedure :: get_reflections         

     ! Utilities
     procedure :: make_reflections
     procedure :: calc_2theta
     procedure :: LPcorr
     procedure :: sfactor
     procedure :: print => print_crystal

     ! Check existence of properties
     procedure :: exist_spg

   end type crystal_phase_type

   CONTAINS
  
   logical function exist_spg(crystal)
   class(crystal_phase_type), intent(in) :: crystal
   exist_spg = crystal%spg%num /= 0
   end function exist_spg

!----------------------------------------------------------------------------------------------

   subroutine set_atoms(crystal,at)
   USE atom_type_util
   class(crystal_phase_type), intent(inout)             :: crystal
   type(atom_type), dimension(:), allocatable, optional :: at
   call copia_atoms(crystal%at,at)
   end subroutine set_atoms
   
!----------------------------------------------------------------------------------------------

   subroutine set_bonds(crystal,bonds)
   USE connect_mod
   class(crystal_phase_type), intent(inout)   :: crystal
   type(bond_type), dimension(:), allocatable :: bonds
!
   call copia_legm(crystal%bonds,bonds)
!
   end subroutine set_bonds
   
!----------------------------------------------------------------------------------------------

   subroutine set_spg(crystal,spg)
   class(crystal_phase_type), intent(inout) :: crystal
   type(spaceg_type), intent(in)            :: spg
!
   if (spg%undef()) return
   crystal%spg = spg
   crystal%has_spg = .true.
!
   end subroutine set_spg

!----------------------------------------------------------------------------------------------

   subroutine set_unit_cell(crystal,cell)
   class(crystal_phase_type), intent(inout) :: crystal
   type(cell_type), intent(in)              :: cell
!
   crystal%cell = cell
   crystal%has_cell = .true.
!
   end subroutine set_unit_cell

!----------------------------------------------------------------------------------------------

   subroutine get_reflections(crystal,ref)
   USE reflection_type_util
   class(crystal_phase_type), intent(inout)                        :: crystal
   type(reflection_type), dimension(:), allocatable, intent(inout) :: ref
!
   call copy_ref(ref,crystal%ref)
!
   end subroutine get_reflections

!----------------------------------------------------------------------------------------------

   subroutine set_symmetry(crystal,spg,cell)
   class(crystal_phase_type), intent(inout) :: crystal
   type(spaceg_type), intent(in)            :: spg
   type(cell_type), intent(in)              :: cell
   call crystal%set_spg(spg)
   call crystal%set_unit_cell(cell)
   end subroutine set_symmetry  

!----------------------------------------------------------------------------------------------

   subroutine set_reflections(crystal,ref)
   USE reflection_type_util
   class(crystal_phase_type), intent(inout)                     :: crystal
   type(reflection_type), dimension(:), allocatable, intent(in) :: ref
!
   call copy_ref(crystal%ref,ref)
!
   end subroutine set_reflections

!----------------------------------------------------------------------------------------------

   subroutine set_scattering(crystal,elem)
   USE elements
   class(crystal_phase_type), intent(inout)      :: crystal
   type(element_type), dimension(:), allocatable :: elem
   call copy_elem(crystal%elem,elem)
   end subroutine set_scattering

!----------------------------------------------------------------------------------------------

   subroutine set_ycalc(crystal,ycalc)
   USE arrayutil
   class(crystal_phase_type), intent(inout) :: crystal
   real, dimension(:), allocatable          :: ycalc
   call copy_array(crystal%yc,ycalc)
   end subroutine set_ycalc

!----------------------------------------------------------------------------------------------

   subroutine make_reflections(crystal,thmin,thmax,wave)
   USE reflection_type_util
   class(crystal_phase_type), intent(inout) :: crystal
   real, intent(in)                         :: thmin,thmax
   real, intent(in)                         :: wave
   integer                                  :: nref
!
   if (.not.crystal%has_cell) return
   if (.not.crystal%has_spg) return
   call create_reflections(thmin,thmax,crystal%cell%get_par(),crystal%spg,wave,crystal%ref,nref)
   call crystal%calc_2theta(wave)
!
   end subroutine make_reflections

!----------------------------------------------------------------------------------------------

   subroutine calc_2theta(crystal,wave)
   USE reflection_type_util
   class(crystal_phase_type), intent(inout) :: crystal
   real, intent(in)                         :: wave
   call calculate_2theta(crystal%ref,crystal%cell%get_par(),wave)
   end subroutine calc_2theta

!----------------------------------------------------------------------------------------------

   subroutine LPcorr(crystal,neutro,nsyn)
   USE reflection_type_util
   class(crystal_phase_type), intent(inout) :: crystal
   integer, intent(in)                      :: neutro,nsyn
   crystal%ref%lp = LPcorrection(crystal%ref%tthd,neutro,nsyn)
   end subroutine LPcorr

!----------------------------------------------------------------------------------------------

   subroutine sfactor(crystal)
   !USE sfacmod
   USE atom_type_util
   USE reflection_type_util
   class(crystal_phase_type), intent(inout) :: crystal
!
   if (.not.crystal%has_cell) return
   if (numatomtype(crystal%at) == 0) return
   if (numelem(crystal%elem) == 0) return
   if (numrefl(crystal%ref) == 0) return

   call sfcalc(crystal%ref,crystal%at,crystal%spg,crystal%elem)
!
   end subroutine sfactor

!----------------------------------------------------------------------------------------------

   subroutine set_crystal(crystal,at,bonds,pname,spg,cell,elem)
   USE atom_type_util
   USE connect_mod
   USE elements
   class(crystal_phase_type), intent(inout)                            :: crystal
   type(atom_type), dimension(:), allocatable, optional                :: at
   type(bond_type), dimension(:), allocatable, optional                :: bonds
   type(spaceg_type), intent(in), optional                             :: spg
   type(cell_type), intent(in), optional                               :: cell
   type(element_type), dimension(:), allocatable, intent(in), optional :: elem
   character(len=*), intent(in), optional                              :: pname
!
   if (present(at)) then
       call crystal%set_atoms(at)
   endif
!
   if (present(bonds)) then
       call crystal%set_bonds(bonds)
   endif
!
   if (present(pname)) then
       crystal%phase_name = trim(pname)
   endif
!
   if (present(spg) .and. present(cell)) then
       call crystal%set_symmetry(spg,cell)
   endif
!
   if (present(elem)) then
       call crystal%set_scattering(elem)
   endif
!
   end subroutine set_crystal

!----------------------------------------------------------------------------------------------

   subroutine print_crystal(crystal,kpr,opt)
   USE atom_type_util
   USE spginfom
   USE ccryst
   USE reflection_type_util
   class(crystal_phase_type), intent(in)  :: crystal
   integer, intent(in)                    :: kpr
   character(len=*), intent(in), optional :: opt
   character(len=:), allocatable          :: sopt
   if (present(opt)) then
       sopt = opt
   else
       sopt = 'SCA'
   endif

   if (index(sopt,'S') > 0) then
       call crystal%spg%prn(kpr,short=.true.)
       write(kpr,'(/)')
   endif

   if (index(sopt,'C') > 0) then
       call print_cell(crystal%cell%get_par(),kpr=kpr)
   endif

   if (index(sopt,'A') > 0) then
       call print_atoms(crystal%at,kpr=kpr)
   endif

   if (index(sopt,'R') > 0) then
       call write_reflections(jfile=kpr,refl=crystal%ref,code=5)
   endif

   if (index(sopt,'E') > 0) then
       call print_elements(crystal%elem,0)
   endif
   end subroutine print_crystal

!----------------------------------------------------------------------------------------------

   integer function numphase(phase)
   type(crystal_phase_type), dimension(:), allocatable, intent(in) :: phase
   if (allocated(phase)) then
       numphase = size(phase)
   else
       numphase = 0
   endif
   end function numphase

!----------------------------------------------------------------------------------------------

   subroutine resize_phases(phase,n)
!
!  Rialloca ad n un vettore reale.
!  Se savevet = .true. o non esiste si salva il suo contenuto.
!
   type(crystal_phase_type), allocatable, intent(inout) :: phase(:)
   integer, intent(in)                                  :: n
   integer                                              :: nv
   type(crystal_phase_type), allocatable                :: vsav(:)
   integer                                              :: nsav
!
!  se n = 0 (riallocazione a 0): dealloca ed esci
   if (n == 0) then
       if (allocated(phase)) deallocate(phase)
       return
   endif
!
   if (.not.allocated(phase)) then
       allocate(phase(n))
   else
!
       nv = size(phase)
!
!      nsav contiene qual è la porzione di phase da salvare
       select case(nv-n)
         case (1:)       ! compatta x ad n
           nsav = n
         case (:-1)      ! espandi x ad n
           nsav = nv
         case (0)
           return        ! n=nv non fare niente
       end select
       allocate(vsav(n))
       vsav(:nsav) = phase(:nsav)
       call move_alloc(vsav,phase)
   endif
!
   end subroutine resize_phases

!----------------------------------------------------------------------------------------------------

   subroutine new_phases(vetr,n)
!
!  Create new atoms
!
   type(crystal_phase_type), allocatable, intent(inout) :: vetr(:)
   integer, intent(in)                         :: n

   if (n < 0) return
   if (numphase(vetr) /= n) then
       if (allocated(vetr))deallocate(vetr)
       if (n > 0) allocate(vetr(n))
   endif

   end subroutine new_phases

!----------------------------------------------------------------------------------------------------

   subroutine clear_phases(vetr)
!
!  Delete all atoms
!
   type(crystal_phase_type), allocatable, intent(inout) :: vetr(:)

   if (allocated(vetr)) deallocate(vetr)

   end subroutine clear_phases

END MODULE crystal_phase
